@@ -1,9 +1,13 @@
+from __future__ import unicode_literals
+
 from itertools import count
+from django.utils.deconstruct import deconstructible
 
 
+@deconstructible
 class Token(object):
     """
-    Hellper for tree-like flow token management
+    Helper for tree-like flow token management.
 
     We follow basic strategy for flow split/join
 
@@ -15,19 +19,27 @@ class Token(object):
     - each join removes corresponding split token addition, so 'start/3_4/7_4'
       becomes 'start/3_4'
     """
+
     def __init__(self, token):
+        """
+        Instantiate a new token.
+
+        :param token: str
+        """
         self.token = token
 
     def is_split_token(self):
+        """True, if it is a token of parallel task."""
         return '/' in self.token
 
     def get_base_split_token(self):
-        """
-        Returns token before split happens
-        """
+        """Return token before last split happens."""
         return Token(self.token.rsplit('/', 1)[0])
 
-    def get_common_split_prefix(self):
+    def get_common_split_prefix(self, join_token, task_pk):
+        """Common prefix for tokens."""
+        if self == join_token:
+            return '{}/{}_'.format(self.token, task_pk)
         return '{}_'.format(self.token.rsplit('_', 1)[0])
 
     def __str__(self):
@@ -45,5 +57,6 @@ class Token(object):
 
     @classmethod
     def split_token_source(cls, prev_token, task_pk):
+        """Span a set of uniq tokens with common prefix."""
         for n in count(1):
-            yield "{}/{}_{}".format(prev_token, task_pk, n)
+            yield Token("{}/{}_{}".format(prev_token, task_pk, n))
